import { __awaiter, __decorate } from "tslib";
import { Connect, ConnectConfig, defaultConnectConfig } from '@genesislcap/foundation-comms';
import { EventEmitter } from '@genesislcap/foundation-events';
import { App } from '@genesislcap/foundation-shell/app';
import { configureDesignSystem } from '@genesislcap/foundation-ui';
import { baseLayerLuminance, customElement, Container, DefaultRouteRecognizer, DOM, GenesisElement, inject, observable, Registration, StandardLuminance, } from '@genesislcap/web-core';
import * as Components from '../components';
import { MainRouterConfig } from '../routes';
import { Store } from '../store';
import designTokens from '../styles/design-tokens.json';
import { MainStyles as styles } from './main.styles';
import { DynamicTemplate as template, LoadingTemplate, MainTemplate } from './main.template';
const name = 'foundation-console-app';
/**
 * @fires store-connected - Fired when the store is connected.
 * @fires store-ready - Fired when the store is ready.
 */
let MainApplication = class MainApplication extends EventEmitter(GenesisElement) {
    constructor() {
        super(...arguments);
        this.ready = false;
        this.data = null;
    }
    connectedCallback() {
        const _super = Object.create(null, {
            connectedCallback: { get: () => super.connectedCallback }
        });
        return __awaiter(this, void 0, void 0, function* () {
            this.registerDIDependencies();
            _super.connectedCallback.call(this);
            this.addEventListeners();
            this.readyStore();
            yield this.loadRemotes();
            DOM.queueUpdate(() => {
                configureDesignSystem(this.provider, designTokens);
            });
        });
    }
    disconnectedCallback() {
        super.disconnectedCallback();
        this.removeEventListeners();
        this.disconnectStore();
    }
    onDarkModeToggle() {
        baseLayerLuminance.setValueFor(this.provider, baseLayerLuminance.getValueFor(this.provider) === StandardLuminance.DarkMode
            ? StandardLuminance.LightMode
            : StandardLuminance.DarkMode);
    }
    loadRemotes() {
        return __awaiter(this, void 0, void 0, function* () {
            const { registerComponents } = Components;
            yield registerComponents();
            this.ready = true;
        });
    }
    selectTemplate() {
        return this.ready ? MainTemplate : LoadingTemplate;
    }
    registerDIDependencies() {
        this.container.register(Registration.transient(DefaultRouteRecognizer, DefaultRouteRecognizer), 
        /**
         * Register custom configs for core services and micro frontends. Note this can also be done via
         * provideDesignSystem().register(...) if preferred.
         *
         * Registration.instance<CredentialManagerConfig>(CredentialManagerConfig, {
         *  ...defaultCredentialManagerConfig,
         *  cookie: {
         *    ...defaultCredentialManagerConfig.cookie,
         *    path: '/login',
         *  },
         * }),
         */
        Registration.instance(ConnectConfig, Object.assign(Object.assign({}, defaultConnectConfig), { connect: Object.assign(Object.assign({}, defaultConnectConfig.connect), { heartbeatInterval: 15000 }) })));
    }
    addEventListeners() {
        this.addEventListener('store-connected', this.store.onConnected);
    }
    removeEventListeners() {
        this.removeEventListener('store-connected', this.store.onConnected);
    }
    readyStore() {
        // @ts-ignore
        this.$emit('store-connected', this);
        this.$emit('store-ready', true);
    }
    disconnectStore() {
        this.$emit('store-disconnected');
    }
};
__decorate([
    App
], MainApplication.prototype, "app", void 0);
__decorate([
    Connect
], MainApplication.prototype, "connect", void 0);
__decorate([
    Container
], MainApplication.prototype, "container", void 0);
__decorate([
    Store
], MainApplication.prototype, "store", void 0);
__decorate([
    inject(MainRouterConfig)
], MainApplication.prototype, "config", void 0);
__decorate([
    observable
], MainApplication.prototype, "provider", void 0);
__decorate([
    observable
], MainApplication.prototype, "ready", void 0);
__decorate([
    observable
], MainApplication.prototype, "data", void 0);
MainApplication = __decorate([
    customElement({
        name,
        template,
        styles,
    })
], MainApplication);
export { MainApplication };
